/*
* usr_401.c -  process io functions needed for co_401.c 
*
* Copyright (c) 2013 emtas GmbH
*-------------------------------------------------------------------
* $Id: usr_401.c,v 1.1 2013/06/25 08:09:02 oe Exp oe $
*
*
*-------------------------------------------------------------------
* Changelog:
*
*
*/


/********************************************************************/
/**
* \brief Implements hardware access to fulfill co_401.c requirements
*
* These functions are called by the CANopen library module co_401
* which implements the CiA 401 profile behaviour.
* The user has to register the functions to the CANopen stack
* before the stack is activated.
*
* \file usr_401.c
*
*/


/* header of standard C - libraries
---------------------------------------------------------------------------*/
// #include <stddef.h>
#include <stdio.h>      /* debug using printf() */
#include <stdlib.h>     /* system() */

/* header of project specific types
---------------------------------------------------------------------------*/
#include <gen_define.h>


#ifdef CO_PROFILE_401
#include <co_datatype.h>
#include <co_timer.h>

#include <usr_401.h>

#include <DAVE3.h>			//Declarations from DAVE3 Code Generation (includes SFR declaration)

#if defined(XMC4500)
/* if Hexagon extension Board is attached with 8 I2C controlled LEDs */
# include <i2c.h>
#endif

void ADC0_Init(void);
void I2C_Transfer_Init(void);

/***************************************************************************/
/**
* \brief - initialize hardware used by 401 commands
*
* I2C is used to control the 8bit expander on the Hexagon COM module
*
* Partly the code is derived from the Dave3  example I2C001_Example1.
*  
*
* \return void
*
*/
void p401HardwareInit(void)
{

	/* initalisation is done in DAVE_Init() at start up */
#if UC_ID == 4502

    I2C_Transfer_Init();

#elif UC_ID == 4206
	/* initalisation of BUTTON1 is done in DAVE_Init() at start up */
	/* Initialize and start ADC */
	ADC0_Init();
#else
    ;
#endif
}

#if 0
I2C001_WriteData \
                  (const I2C001Handle_type* I2CHandle,const I2C001_DataType* Data)
#endif

#if UC_ID == 4502
void I2C_Transfer_Init(void)
{
I2C001_DataType data;

	/*  I2C sequence to set all 8 GPIO of the PC 9502 as output
	*  Transmission by the master with start condition,
	*  I2C WRITE condition and slave address
	*/
	data.Data1.TDF_Type = I2C_TDF_MStart;
	data.Data1.Data = (0x98 | I2C_WRITE);
	I2C001_WriteData(&I2C001_Handle0, &data);

	// Write to the direction register
	data.Data1.TDF_Type = I2C_TDF_MTxData;
	data.Data1.Data = 0x50;
	I2C001_WriteData(&I2C001_Handle0, &data);

	//   Write data to configure one bit as output
	data.Data1.TDF_Type = I2C_TDF_MTxData;
	data.Data1.Data = 0xFF;
	I2C001_WriteData(&I2C001_Handle0, &data);

	/*  I2C sequence to the turn on every other LED:
	 *  write into the State register 0xAA */
	// Transmit repeated start condition and slave address
	data.Data1.TDF_Type = I2C_TDF_MRStart;
	data.Data1.Data = 0x98 | I2C_WRITE;  // address of the GPI expander PCA 9502
	I2C001_WriteData(&I2C001_Handle0, &data);

	// Write data to address the State register
	data.Data1.TDF_Type = I2C_TDF_MTxData;
	data.Data1.Data = 0x58; // address of  state register in the GPI expander
	I2C001_WriteData(&I2C001_Handle0, &data);

	// Write data to the state register to turn all LED off
	data.Data1.TDF_Type = I2C_TDF_MTxData;
	data.Data1.Data = 0xff; //  turn any other LED on, LED is on when writing 0
	I2C001_WriteData(&I2C001_Handle0, &data);

	// Write data to the state register to turn all LED off
	data.Data1.TDF_Type = I2C_TDF_MTxData;
	data.Data1.Data = 0x00; //  turn any other LED on, LED is on when writing 0
	I2C001_WriteData(&I2C001_Handle0, &data);
}


/* write the 8 but value to the GPIO port
 *
 * because the LEDs are driven by pull down
 * the value is inverted here in order
 * to switch a LED on with a logical 1.
 */
void setLED(uint8_t led)
{
I2C001_DataType data;

    // led ^= 0xFFFF;
    // Write data to the state register to turn some other LED on
    data.Data1.TDF_Type = I2C_TDF_MTxData;
    data.Data1.Data = led; //  turn any other LED on, LED is on when writing 0
    I2C001_WriteData(&I2C001_Handle0, &data);
}
#endif

#if UC_ID == 4206
/**
 * @brief This function is to initialize the VADC peripheral to continuous conversion with background source\n
 * Channel 1 in group 0 takes part in the scan sequence. The result is sent to RESREG[0]\n
 * @param[in]   none \n
 * @return none\n
 *
 * <b>Reentrancy:  no</b><BR>
 *
 */
void ADC0_Init(void)
{
	SCU_CLK->CGATCLR0 = SCU_CLK_CGATCLR0_VADC_Msk;

	SCU_RESET->PRCLR0 |= ((1 << SCU_RESET_PRSET0_VADCRS_Pos) & SCU_RESET_PRSET0_VADCRS_Msk);
                                        // De-assert reset: VADC Reset Clear Reset Clear = 1

	//Global ADC Initialization

	VADC->CLC         =  ((0 << VADC_CLC_DISR_Pos) & VADC_CLC_DISR_Msk) |
                                        // DISR = 0: Module Disable Request Bit
                                        // 0B On request: enable the module clock
                       	 ((0 << VADC_CLC_DISS_Pos) & VADC_CLC_DISS_Msk) |
                                        // DISS = 0: Module Disable Status Bit
                                        // 0B Module clock is enabled
                       	 ((0 << VADC_CLC_EDIS_Pos) & VADC_CLC_EDIS_Msk);
                                        // EDIS = 0: Sleep Mode Enable Control
                                        // 0B Sleep mode request is enabled and functional
	// Wait for module clock enabled
	while(VADC->CLC!=0);                  // DISS: Module Disable Status Bit
                                        //  0B Module clock is enabled

	VADC->GLOBCFG     =  ((6 << VADC_GLOBCFG_DIVA_Pos) & VADC_GLOBCFG_DIVA_Msk) |
                                        // DIVA = 6: Divider Factor for the Analog Internal Clock
                                        // fADCI = fADC / (DIVA + 1) = fADC / 7
                       	 ((1 << VADC_GLOBCFG_DIVWC_Pos) & VADC_GLOBCFG_DIVWC_Msk);
                                        // DIVWC = 1: Write Control for Divider Parameters
                                        // Bitfields DIVA, DCMSB, DIVD can be written

	//Arbitration
	VADC_G0->ARBCFG   =  ((VADC_G0->ARBCFG &
                       	 ~(VADC_G_ARBCFG_ANONC_Msk | VADC_G_ARBCFG_ARBRND_Msk | VADC_G_ARBCFG_ARBM_Msk)) |
                                        // Clear relevant bits
                       	 (((3 << VADC_G_ARBCFG_ANONC_Pos) & VADC_G_ARBCFG_ANONC_Msk) |
                                        // ANONS = 11B: Analog Converter Control: Normal Operation
                                        //  The converter is active, conversions are started immediately.
                                        //  Requires no wakeup time.
                       	 ((0 << VADC_G_ARBCFG_ARBRND_Pos) & VADC_G_ARBCFG_ARBRND_Msk)|
                                        // ARBRND = 00B: Arbitration Round Length
                                        //  00B: 4 arbitration slots per round (tARB = 4 / fADCD)
                         ((0 << VADC_G_ARBCFG_ARBM_Pos) & VADC_G_ARBCFG_ARBM_Msk)));
                                        // ARBM = 0B: Arbitration Mode: The arbiter runs permanently

	VADC_G0->ARBPR    =  ((VADC_G0->ARBPR &
                         ~(VADC_G_ARBPR_ASEN2_Msk | VADC_G_ARBPR_PRIO0_Msk)) |
                                        // clear relevant bits
                         (((1 << VADC_G_ARBPR_PRIO0_Pos) & VADC_G_ARBPR_PRIO0_Msk) |
                                        // PRIO0 = 1: Priority of Request Source x (x = 0)
                                        //  Arbitration priority of request source x (in slot x)
                         ((1 << VADC_G_ARBPR_ASEN2_Pos) & VADC_G_ARBPR_ASEN2_Msk)));
                                        // ASEN2 = 1: Arbitration Slot 2 Enable

	//Conversion result storage
	VADC_G0->CHCTR[4] =  ((0x1 << VADC_G_CHCTR_RESREG_Pos) & VADC_G_CHCTR_RESREG_Msk);
                                        // Channel 1 is directed to RESREG1
                                        //  0001B Store result in group result register GxRES1
                                        //  = VADC_G0->RES[1]

	//BACKGROUND SOURCE
	VADC->BRSSEL[0]   =  (1UL << 4) ;
                                        // CHSELG1 = 1: Channel Selection Group 0: Channel 4
                                        //  1B This channel is part of the scan sequence

	VADC->BRSCTRL     =  ((0 << VADC_BRSCTRL_XTSEL_Pos) & VADC_BRSCTRL_XTSEL_Msk) |
                                        // XTSEL = 0: External Trigger Input Selection
                                        //   not used: see XTMODE
                         ((0 << VADC_BRSCTRL_XTMODE_Pos) & VADC_BRSCTRL_XTMODE_Msk) |
                                        // XTMODE = 0: Trigger Operating Mode: No external trigger
                         ((1 << VADC_BRSCTRL_XTWC_Pos) & VADC_BRSCTRL_XTWC_Msk) |
                                        // XTWC = 1: Write Control for Trigger Configuration
                                        //  Bitfields XTMODE and XTSEL can be written
                         ((0 << VADC_BRSCTRL_GTSEL_Pos) & VADC_BRSCTRL_GTSEL_Msk) |
                                        // GTSEL = 0: Gate Input Selection
                         ((1 << VADC_BRSCTRL_GTWC_Pos) & VADC_BRSCTRL_GTWC_Msk);
                                        // GTWC = 1: Write Control for Gate Configuration
                                        //  Bitfield GTSEL can be written

	VADC->BRSMR       =  ((1 << VADC_BRSMR_ENGT_Pos) & VADC_BRSMR_ENGT_Msk) |
                                        // ENGT = 1: Enable Gate: 01B Conversion requests are
                                        //  issued if at least one pending bit is set
                       	 ((0 << VADC_BRSMR_ENTR_Pos) & VADC_BRSMR_ENTR_Msk) |
                                        // ENTR = 0: Enable External Trigger: 0B External trigger disabled
                         ((0 << VADC_BRSMR_ENSI_Pos) & VADC_BRSMR_ENSI_Msk) |
                                        // ENSI = 0: Enable Source Interrupt: 0B No request source interrupt
                         ((1 << VADC_BRSMR_SCAN_Pos) & VADC_BRSMR_SCAN_Msk) |
                                        // SCAN = 1: Autoscan Enable: 1B Autoscan functionality enabled:
                                        //  a request source event automatically generates a load event
                         ((0 << VADC_BRSMR_LDM_Pos) & VADC_BRSMR_LDM_Msk) |
                                        // LDM = 0: Autoscan Source Load Event Mode:
                                        //  0B Overwrite mode: Copy all bits from the select registers to the
                                        //  pending registers upon a load event
                         ((1 << VADC_BRSMR_LDEV_Pos) & VADC_BRSMR_LDEV_Msk);
                                        // LDEV = 1: Generate Load Event: 1B A load event is generated
                                        //  it is important to have a load event in initialization!
                                        //  after that autoscan takes control
}
#endif

/***************************************************************************/
/**
* \brief - set an 8 bit digital output - printf()
*
* This simple version uses only printf() to print the information
* to the stdout of the process.
*
* This function has to be registered using coEventRegister_401()
* \code
*   coEventRegister_401(NULL, byte_out_printfInd, NULL, NULL)
* \endcode
*
* \return void
*
*/
void byte_out_printfInd(
        UNSIGNED8 port,         /**< selected 8bit port */
        UNSIGNED16 outVal       /**< output value */
    )
{
#if 0
    printf("digout port:%2d, value:0x%02x, %3d\n",
            port, outVal, outVal);
#endif
}


/***************************************************************************/
/**
* \brief - set an 8 bit digital output - process image
*
* This version uses  a virtual file system
* where digout ports are files named like the port number
* under the directory digout.
* The port value is written there as decimal value.
*
* This function has to be registered using coEventRegister_401()
* \code
*   coEventRegister_401(NULL, byte_out_piInd, NULL, NULL)
* \endcode
*
* \return void
*
*/

void byte_out_piInd(
        UNSIGNED8 port,         /**< selected 8bit port */
        UNSIGNED16 outVal       /**< output value */
    )
{

#if UC_ID == 4502
	/* Infineon Hexagon Kit with COM expansion
     * the expansion board has an I2C controlled 8 bit port with 8 LEDs.
     * But only one. */
    if( port == 1) {
        /* only port 1 ia available */
       setLED((UNSIGNED8)(outVal));
    }
#endif
}

/***************************************************************************/
/**
* \brief - read an 8 bit digital input - process image
*
* This version uses  a virtual file system
* where digin ports are files named like the port number
* under the directory digin.
* The port value is read there as decimal value.
*
* This function has to be registered using coEventRegister_401()
* \code
*   coEventRegister_401(byte_in_piInd, NULL, NULL, NULL)
* \endcode
*
* \return void
*
*/

UNSIGNED8 byte_in_piInd(
        UNSIGNED8 port,         /**< selected 8bit port */
        UNSIGNED8 filter        /**< port filter value */
    )
{
// FILE *fp;
// char file[100];
UNSIGNED8 value;
bool status = 0;

    (void)filter;           /* not used in this version */

    value = 0;
    if (port == 1) {
		/* this function reads only one pin status and delivers true or false */
#if UC_ID == 4206
		status = IO001_ReadPin(IO001_Handle0);
#else
		status = 0;
#endif
		/* map the pin to one bit in the byte returned later */

		/* on the XMC4200 board the BUTTON1 pressed delivers false */
		if(status == 0)
			value = 1;
		else
			value = 0;
	}
#if 0
    /* code used with the linux example using
	 a file based process image */
    snprintf(file, 100, "digin/%d", port);
    fp = fopen(file, "r");
    if (fp != NULL) {
        fgets(file, 100, fp);
        value = atoi(file);
        fclose(fp);
    } else {
        printf("process image at \"%s\" not found\n", file);
        exit(14);
    }
#endif
    /* printf("Input from port %d, val=0x%02x\n", port, value); */
    return value;
}



/***************************************************************************/
/**
* \brief - set an 16bit analog output - printf()
*
* This simple version uses only printf() to print the information
* to the stdout of the process.
*
* This function has to be registered using coEventRegister_401()
* \code
* coEventRegister_401(NULL, NULL, NULL, analog_out_printfInd)
* \endcode
*
* \return void
*
*/
void analog_out_printfInd(
        UNSIGNED8 port,         /**< selected 16bit analog channel */
        INTEGER16 outVal        /**< output value */
    )
{
#if 0
	printf("analog out port:%2d, value:%5d\n",
            port, outVal);
#endif
}

/***************************************************************************/
/**
* \brief - set an 16bit analog output - process image
*
* This version uses a virtual file system
* where anout ports are files named like the port number
* under the directory anout.
* The port value is written there as decimal value.
*
* \note 
* All analog channels are 16 bit integer.
* If hardware has a lower resolution, use the most left aligned bits.
*
* This function has to be registered using coEventRegister_401()
* \code
*   coEventRegister_401(NULL, NULL, NULL, analog_out_piInd)
* \endcode
*
*  \return void
*
*/
void analog_out_piInd(
        UNSIGNED8 port,         /**< selected 16bit analog channel */
        INTEGER16 outVal        /**< output value */
    )
{
#if 0
char command[100];

    printf("analog out port:%2d, value:%5d\n",
            port, outVal);
#if 0
    snprintf(command, 100, "/bin/echo %d > anout/%d", outVal, port);
    system(command);
#endif
#endif
}

/***************************************************************************/
/**
* \brief - read an 16 bit analog input - process image
*
* This version uses  a virtual file system
* where anin ports are files named like the port number
* under the directory anin.
* The port value is read there as decimal value.
*
* \note 
* All analog channels are 16 bit integer.
* If hardware has a lower resolution, use the most left aligned bits.
*
* This function has to be registered using coEventRegister_401()
* \code
*   coEventRegister_401(NULL, NULL, analog_in_piInd, NULL)
* \endcode
*
* \return void
*
*/

INTEGER16 analog_in_piInd(
        UNSIGNED8 port          /**< selected 8bit port */
    )
{
// FILE *fp;
// char file[100];
INTEGER16 value;
unsigned long adc_result;

    value = 0;

    /* XMC4200 has only one channel on board */
    if (port != 1) return value;



	do{
		adc_result =  VADC_G0->RES[1];
	} while (!(adc_result >> VADC_G_RES_VF_Pos));
									// wait until ADC result register 0 value is valid
									// VADC.G0RES1.VF = 1: Valid Flag
									// Indicates a new result in bitfield RESULT or bit FCR.
									// 1B = Bitfield RESULT has been updated with new
									// result value and has not yet been read, or bit
									// FCR has been updated

	adc_result &= 0xFFF;           // mask ADC result
								   // toggle P2.1 (toggle LED) using GPIO.h macros

	value = (UNSIGNED16)adc_result;

#if 0
    snprintf(file, 100, "anin/%d", port);
    fp = fopen(file, "r");
    if (fp != NULL) {
        fgets(file, 100, fp);
        value = atoi(file);
        fclose(fp);
    } else {
        printf("process image at \"%s\" not found\n", file);
        exit(14);
    }
#endif
    // printf("Input from port %2d, val=%7d\n", port, value);
    return value;
}



#endif /* CO_PROFILE_401 */
/* vim: set ts=4 sw=4 spelllang=en : */
